import json

import numpy as np

from pyms.mlpy import readmda, writemda64

processor_name = 'pyms.merge_burst_parents'
processor_version = '0.11'


def merge_burst_parents(*, firings, metrics, firings_out):
    """
    Identify clusters labeled as merge parents in the metrics file, perform the merge, and save the new firings.mda
    Based on p_create_label_map / apply_label_map by JFM and JC

    Parameters
    ----------
    firings : INPUT
        Path of input firings mda file
    metrics : INPUT
        Path of metrics json file to be used for generating the merge map
    firings_out : OUTPUT
        updated firings, post-merge


    """

    merge_map = []
    firings = readmda(firings)

    # Load json
    with open(metrics) as metrics_json:
        metrics_data = json.load(metrics_json)

    # Iterate through all clusters and identify clusters to merge
    for idx in range(len(metrics_data['clusters'])):

        # Check if burst parent exists
        if metrics_data['clusters'][idx]['metrics']['bursting_parent']:
            merge_map.append([metrics_data['clusters'][idx]['metrics']['bursting_parent'],
                              metrics_data['clusters'][idx]['label']])
        else:
            merge_map.append([metrics_data['clusters'][idx]['label'],
                              metrics_data['clusters'][idx]['label']])  # otherwise, map to itself!

    merge_map = np.reshape(merge_map, (-1, 2))
    # Assure input is sorted
    merge_map = merge_map[np.argsort(merge_map[:, 0])]
    print(merge_map)

    # Propagate merge pairs to lowest label number
    for idx, label in enumerate(merge_map[:, 1]):
        print(label)
        merge_map[np.where(merge_map[:, 0] == label)[0],
                  0] = merge_map[idx, 0]  # Input should be sorted

    # Apply label map
    print(range(merge_map.shape[0]))
    for merge_pair in range(merge_map.shape[0]):
        print(merge_pair)
        firings[2, np.where(firings[2, :] == merge_map[merge_pair, 1])[
            0]] = merge_map[merge_pair, 0]

    # Write remapped firings
    return writemda64(firings, firings_out)


merge_burst_parents.name = processor_name
merge_burst_parents.version = processor_version
merge_burst_parents.author = 'AKGillespie'



import json
import os

parent_path = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

processor_name = 'pyms.add_curation_tags'
processor_version = '0.11'


def add_curation_tags(*, metrics, metrics_tagged, firing_rate_thresh=0, isolation_thresh=0, noise_overlap_thresh=1, peak_snr_thresh=0, mv2file=[]):
    """
    Add tags to the metrics file to reflect which clusters should be rejected based on curation criteria
    Based on create/apply label map by J Chung and J Magland

    Parameters
    ----------
    metrics : INPUT
        Path of metrics json file to add tags
    metrics_tagged : OUTPUT
        Path of metricsjson which has been updated with cluster tags
        ...

    firing_rate_thresh : float64
        (Optional) firing rate must be above this
    isolation_thresh : float64
        (Optional) isolation must be above this
    noise_overlap_thresh : float64
        (Optional) noise_overlap_thresh must be below this
    peak_snr_thresh : float64
        (Optional) peak snr must be above this
    mv2file : string
        (Optional) if tags have already been added, update new metrics file with them
        ...
    """
    # Load json
    with open(metrics) as metrics_json:
        metrics_data = json.load(metrics_json)

    if mv2file:
        with open(mv2file) as f:
            mv2 = json.load(f)

    # Iterate through all clusters
    for idx in range(len(metrics_data['clusters'])):

        # initialize empty tags key
        if 'tags' not in metrics_data['clusters'][idx]:
            metrics_data['clusters'][idx]['tags'] = []

        # if the mv2 was passed in, use those tags
        if mv2file:
            clustlabel = metrics_data['clusters'][idx]['label']
            metrics_data['clusters'][idx]['tags'] = mv2['cluster_attributes'][str(
                clustlabel)]['tags']

        if metrics_data['clusters'][idx]['metrics']['firing_rate'] < firing_rate_thresh or \
                metrics_data['clusters'][idx]['metrics']['isolation'] < isolation_thresh or \
                metrics_data['clusters'][idx]['metrics']['noise_overlap'] > noise_overlap_thresh or \
                metrics_data['clusters'][idx]['metrics']['peak_snr'] < peak_snr_thresh:

            # Add "rejected" tag to cluster metrics (if it isnt already there)
            if 'mua' not in metrics_data['clusters'][idx]['tags']:
                metrics_data['clusters'][idx]['tags'] += ['mua']
            # if the cluster was formerly tagged as accepted, remove that
            if 'accepted' in metrics_data['clusters'][idx]['tags']:
                metrics_data['clusters'][idx]['tags'].remove('accepted')

    # Write out updated metrics json
    with open(metrics_tagged, 'w') as f:
        json.dump(metrics_data, f, sort_keys=True, indent=4)


add_curation_tags.name = processor_name
add_curation_tags.version = processor_version
add_curation_tags.author = 'AKGillespie'



import os
import sys

from franklab_mstaggedcuration.p_add_curation_tags import add_curation_tags
from franklab_mstaggedcuration.p_merge_burst_parents import merge_burst_parents
from pyms.mlpy import ProcessorManager

parent_path = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

PM = ProcessorManager()

PM.registerProcessor(add_curation_tags)
PM.registerProcessor(merge_burst_parents)

if not PM.run(sys.argv):
    exit(-1)



#!/usr/bin/env python3

from setuptools import find_packages, setup
from setuptools.command.develop import develop
from setuptools.command.install import install
import subprocess


def symlink():
    package = 'franklab_mstaggedcuration'
    symlink_cmd = (f'ml-link-python-module {package} '
                   f'`ml-config package_directory`/{package}')
    subprocess.run(symlink_cmd, shell=True)


class PostDevelopCommand(develop):
    """Post-installation for development mode."""

    def run(self):
        symlink()
        develop.run(self)


class PostInstallCommand(install):
    """Post-installation for installation mode."""

    def run(self):
        symlink()
        install.run(self)


INSTALL_REQUIRES = ['numpy']
TESTS_REQUIRE = []

setup(
    name='franklab_mstaggedcuration',
    version='0.1.0.dev0',
    license='',
    description=(''),
    author='',
    author_email='',
    url='',
    packages=find_packages(),
    install_requires=INSTALL_REQUIRES,
    tests_require=TESTS_REQUIRE,
    cmdclass={
        'develop': PostDevelopCommand,
        'install': PostInstallCommand,
    },
)
